@@ -74,6 +74,7 @@ refresh_image (GtkImage * image)
 
 	if (pixbuf == NULL) {
 		g_warning("Unable to load icon from file '%s' because: %s", icon_filename, error == NULL ? "I don't know" : error->message);
+		g_clear_error (&error);
 		gtk_image_clear(image);
 		return;
 	}
@@ -142,7 +143,7 @@ indicator_image_helper_update (GtkImage * image, const gchar * name)
 {
 	g_return_if_fail(name != NULL);
 	g_return_if_fail(name[0] != '\0');
-	g_return_if_fail(image != NULL);
+	g_return_if_fail(GTK_IS_IMAGE(image));
 	gboolean seen_previously = FALSE;
 
 	/* Build us a GIcon */
